// libraries used :
// Adafruit-GFX-Library-master
// Adafruit-ST7735-Library-master
#include <Arduino.h>
#include <Adafruit_GFX.h>    // Core graphics library "Adafruit-GFX-Library-master"
#include <Adafruit_ST7735.h> // Hardware-specific library for ST7735
#include <SPI.h>
#include <SDS011.h>
#include "monimagecouleur.h"  // logo
//
// MQ135 ****************************************************************************
#include <MQUnifiedsensor.h>
#define placa "Arduino NANO"
#define Voltage_Resolution 4.92
#define pin A0 //Analog input 0 of your arduino
#define type "MQ-135" //MQ135
#define ADC_Bit_Resolution 10 // For arduino UNO/MEGA/NANO
#define RatioMQ135CleanAir 3.6//RS / R0 = 3.6 ppm 
float CO2;
// Declare MQ135 Sensor
MQUnifiedsensor MQ135(placa, Voltage_Resolution, ADC_Bit_Resolution, pin, type);
// end MQ135 *****************************************************************************
// Modifications to Adafruit_ST77xx.h becaus ST7735 type 132x162
//-> #define ST7735_TFTWIDTH_128 132  // for 1.44 and mini
//-> #define ST7735_TFTHEIGHT_160 162 // for 1.8" and mini display
//
#define TFT_CS        10
#define TFT_RST        8 // Or set to -1 and connect to Arduino RESET pin
#define TFT_DC 9
// variables
int analogPin = A6;    // mesure voltage 
int indsw = A4;        // switch for display mode
int val;               // current switch value
int indswmem = 0;      // stores last indsw value
float diviseur = 100;
// Time variables
long TempsInit = 0;
long TempsReel = 0;
int TempsDec = 30;
int TempsAff = 0;
// display areas coordonate
int base0 = 20;
int base1 = 80;
int base2 = 160;
int hauteur = 59;
int color;
// SDS011 variables
float p10,p25;
String sp10s,sp25s,sp10st,sp25st,strv;
char p10s[20],p25s[20],p10st[20],p25st[20],trv[20];
bool error = false;
int rx= 4;
int tx= 3;
const int tailtab = 22; // number of measurements to display
int tabps25[tailtab];
int tabps10[tailtab];
int tabco2[tailtab]; // MQ135 values
int tabval;
// Declare SDS011
SDS011 my_sds;
// Declare display. For 1.44" and 1.8" TFT with ST7735 use:
Adafruit_ST7735 tft = Adafruit_ST7735(TFT_CS, TFT_DC, TFT_RST);
// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//                                          setup
// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void setup(void) {
  pinMode(indsw, INPUT); // read display mode
  // MQ135 Set math model to calculate the PPM concentration and the value of constants
  MQ135.setRegressionMethod(1); //_PPM =  a*ratio^b
  MQ135.init(); 
  // MQ135 R0 calculation
  MQ135.setRL(10);
  float calcR0 = 0;
  for(int i = 1; i<=10; i ++) {
    MQ135.update(); // Update data, the arduino will be read the voltage on the analog pin
    calcR0 += MQ135.calibrate(RatioMQ135CleanAir);
  }
  MQ135.setR0(calcR0/10);
  // end MQ135 R0 calculation
  //
  // SDS011
  my_sds.begin(rx,tx);  
  // initialize measurement table
  for (int u = 0; u < 15; u++) { 
    tabps25[u] = random(40);
    tabps10[u] = random(250);
    tabco2[u]  = random(100);  // MQ135
  }
  tft.initR(INITR_BLACKTAB);
  // display logo
  start();
  delay(5000); 
  tft.fillScreen(ST77XX_BLACK);
  //+++++++++++++++++++++++ display top +++++++++++++++++++++++++++
  //   *********
  //   *XXXXXXX*
  //   *XXXXXXX*
  //   *********
  //   *       *
  //   *       *
  //   *********
  //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  aff00(); // display top
  val = digitalRead(indsw);
  //+++++++++++++++++++++++ display bottom +++++++++++++++++++++++
  //   *********
  //   *       *
  //   *       *
  //   *********
  //   *XXXXXXX*
  //   *XXXXXXX*
  //   *********
  //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++  
  switch (val) {
    case 1: 
      aff03(); // display MQ135 values
      break;
    case 0: 
      aff02(); // display SDS011 values  
      break;
  }
}
// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//                                          loop
// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void loop(void) {
  //+++++++++++++++++++++++ change bottom if switch activated +++++++++++++++++++++++
  //   *********
  //   *       *
  //   *       *
  //   *********
  //   *XXXXXXX*
  //   *XXXXXXX*
  //   *********
  //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++  
  val = digitalRead(indsw);
  if (indswmem != val) {
    switch (indswmem) {
      case 1: 
        aff02(); // display bottom
        break;
      case 0: 
        aff03(); // display bottom  
        break;
    }
    indswmem = val;
  }
  //
  // MQ135 read sensor **************
  //
  MQ135.update(); // Update data, the arduino will be read the voltage on the analog pin
  MQ135.setA(110.47); MQ135.setB(-2.862); // Configurate the ecuation values to get CO2 concentration
  CO2 = MQ135.readSensor(); // Sensor will read PPM concentration using the model and a and b values setted before or in the setup
  // end of reading MQ135 ****************
  //
  // Read SDS011
  //
  error = my_sds.read(&p25,&p10);
  if (! error) {
    // shift SDS values to the left, new value is at right border
    for (int u = 0; u < (tailtab-1); u++) {
      tabps25[u] = tabps25[u+1];
      tabps10[u] = tabps10[u+1];
      tabco2[u] = tabco2[u+1];   // MQ135
    }
    tabps25[tailtab-1] = p25;
    tabps10[tailtab-1] = p10;
    tabco2[tailtab-1] = CO2;   // MQ135
    TempsInit = millis();
    TempsDec = 120; // time before next SDS read
    tft.fillScreen(ST77XX_BLACK);
    //+++++++++++++++++++++++ display top +++++++++++++++++++++++++++
    //   *********
    //   *XXXXXXX*
    //   *XXXXXXX*
    //   *********
    //   *       *
    //   *       *
    //   *********
    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    aff00();
    //+++++++++++++++++++++++ display bottom +++++++++++++++++++++++
    //   *********
    //   *       *
    //   *       *
    //   *********
    //   *XXXXXXX*
    //   *XXXXXXX*
    //   *********
    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++   
    switch (val) {
      case 1: 
        aff03(); // display MQ135 values
        break;
      case 0: 
        aff02(); // display SDS011 values 
        break;
    }
  } // end of SDS treatment
  //+++++++++++++++++++++++ display time to next SDS011 info ++++++
  //   *********
  //   *     XX*
  //   *       *
  //   *********
  //   *       *
  //   *       *
  //   *********
    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++   
  TempsReel = (millis() - TempsInit);
  TempsAff = TempsDec - (TempsReel /1000);
  tft.fillRect(71, 0, 70, 20, ST77XX_BLACK); 
  tft.setCursor(80, 2); 
  tft.setTextSize(2);
  tft.setTextColor(ST77XX_WHITE);
  tft.println(String(TempsAff)+"s");
  //+++++++++ MQ135 display CO2 or if voltage under limit info ++++++
  //   *********
  //   *       *
  //   *       *
  //   *********
  //   *     XX*
  //   *       *
  //   *********
  //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++  
  tft.fillRect(69, base1+10, 70, 10, ST77XX_BLACK);
  if (analogRead(analogPin)/diviseur < 6.2) {
    tft.setTextSize(1);
    tft.setTextColor(ST77XX_RED);
    tft.setCursor(70, base1+10);// display voltage low
    tft.print(" BAT LOW!");
  } else {  
    tft.setTextSize(1);
    tft.setTextColor(ST77XX_WHITE);
    tft.setCursor(70, base1+10);// display MQ135 value 
    tft.print("CO2:" + String(CO2+400));
   }
  delay(1000); 
}
  // +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  //                                          Routines
  // +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void start() {
  // display logo
  tft.fillScreen(ST77XX_BLUE);
  int h = 105,w = 80, row, col, buffidx=0;
  for (row=0; row<h; row++) { // For each scanline...
    for (col=0; col<w; col++) { // For each pixel...
      // image is stored as uint16_t, pgm_read_word is used as it uses 16bit address
      tft.drawPixel(col, row, pgm_read_word(monimagecouleur + buffidx));
      buffidx++;
    } // end pixel
  tft.setTextColor(ST77XX_WHITE);
  tft.setTextSize(2);    
  tft.setCursor(6,120);
  tft.print("ACTIVATION");
  }  
}
  // +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  //                                          aff00
  // +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void aff00() { // display top area SDS011
  tft.invertDisplay(false);
  // display PM2.5 value
  tft.fillRect(0, 0, 70, 20, ST77XX_BLUE); 
  tft.setCursor(2, 3); // display counter
  color=ST77XX_GREEN;
  if (p25 > 9) {
     color=ST77XX_RED;
  }
  tft.setTextSize(2);
  tft.setTextColor(color); 
  tft.print(String(p25));
  //
  // Fill area PM2.5
  //
  tft.fillRect(0, base0, 132, hauteur, ST77XX_BLUE);  
  for (int u = 0; u < tailtab; u++) {
      color = ST77XX_GREEN;
      tabval = tabps25[u];      
    if (tabps25[u] > 10) {
      color = ST77XX_RED;
    }
    tft.fillRect(u*6, (base1-tabval), 6, (tabval), color);  
  }
  tft.setCursor(2, base0+2);
  tft.setTextSize(1);
  tft.setTextColor(ST77XX_WHITE);
  tft.println("Q. Part.< 2,5 microns");
  // draw level lines
  tft.setTextSize(1);  
  tft.setCursor(2, base1-10-3);
  tft.print(" >10<");
  tft.drawLine(27, base1-10, 132, base1-10, ST77XX_WHITE); 
  tft.drawLine(0, base1-10, 10, base1-10, ST77XX_WHITE); 
  //
  tft.setCursor(2, base1-20-3);
  tft.print(" >20<");
  tft.drawLine(27, base1-20, 132, base1-20, ST77XX_WHITE); 
  tft.drawLine(0, base1-20, 10, base1-20, ST77XX_WHITE);
  // draw line between two areas
  tft.drawLine(0, base1, 132, base1, ST77XX_MAGENTA);
  tft.drawLine(0, base1+1, 132, base1+1, ST77XX_MAGENTA);
} // end aff00
  // +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  //                                          aff02
  // +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void aff02() { // bottom area SDS011
  tft.invertDisplay(false);
  //
  // display PM10 value
  //
  tft.fillRect(0, base1+2, 70, 20, ST77XX_BLUE); 
  color=ST77XX_GREEN;
  if (p10 > 49) {
     color=ST77XX_RED;
  }
  tft.setTextSize(2);
  tft.setCursor(2, base1+4);
  tft.setTextColor(color); 
  tft.println(String(p10));
  //
  // Fill area PM10
  //
  tft.fillRect(0, base2-hauteur, 132, hauteur, ST77XX_BLUE); 
  for (int u = 0; u < tailtab; u++) {
    color = ST77XX_GREEN;
    tabval = tabps10[u];      
    if (tabps10[u] > 49) {
      color = ST77XX_RED;
    }
  tft.fillRect(u*6, (base2-tabval/5), 6, (tabval/5), color); 
  } 
  tft.setCursor(2, base2-hauteur+2);
  tft.setTextSize(1);
  tft.setTextColor(ST77XX_WHITE);
  tft.println("Q. Part. < 10 microns");
  // drw level lines
  tft.setTextSize(1);    
  tft.setCursor(2, base2-10-3);
  tft.print(" >50<");  
  tft.drawLine(27, base2-10, 132, base2-10, ST77XX_WHITE);
  tft.drawLine(0, base2-10, 10, base2-10, ST77XX_WHITE); 
  //  
  tft.setTextSize(1);    
  tft.setCursor(2, base2-20-3);
  tft.print(" >100<");  
  tft.drawLine(33, base2-20, 132, base2-20, ST77XX_WHITE);
  tft.drawLine(0, base2-20, 10, base2-20, ST77XX_WHITE); 
} // end aff02
  // +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  //                                          aff03
  // +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void aff03() { // bottom area with MQ135-CO2
  tft.invertDisplay(false);
  //
  // display "CO2" 
  //
  tft.setCursor(2, base1+4);
  tft.setTextSize(2);
  tft.setTextColor(ST77XX_WHITE); 
  tft.fillRect(0, base1+2, 70, 20, ST77XX_BLACK); 
  tft.println(" CO2");
  // Fill area MQ135-CO2
  //
  tft.fillRect(0, base2-hauteur, 132, hauteur, ST77XX_BLACK); 
  for (int u = 0; u < tailtab; u++) {
    color = ST77XX_GREEN;
    tabval = tabco2[u];      
    if (tabco2[u] > 20) {
      color = ST77XX_RED;
    }
    if (tabco2[u] > 99) {  // out of display
      tabco2[u] = 100;
    }
    tft.fillRect(u*6, (base2-tabval/2), 6, (tabval/2), color); 
  } 
  tft.setCursor(2, base2-hauteur+2);
  tft.setTextSize(1);
  tft.setTextColor(ST77XX_WHITE);
  tft.println("              (PPM)");
  // drw level lines
  tft.setTextSize(1);    
  tft.setCursor(2, base2-25-3);
  tft.print(" >450<");  
  tft.drawLine(27, base2-25, 132, base2-25, ST77XX_WHITE);
  tft.drawLine(0, base2-25, 10, base2-25, ST77XX_WHITE); 
  //  
  tft.setTextSize(1);    
  tft.setCursor(2, base2-50-3);
  tft.print(" >500<");  
  tft.drawLine(33, base2-50, 132, base2-50, ST77XX_WHITE);
  tft.drawLine(0, base2-50, 10, base2-50, ST77XX_WHITE); 
} // end aff03 MQ135 
